@@ -34,8 +34,6 @@
 #include <stdarg.h>
 #include <locale.h>
 
-#include <ft2build.h>
-#include FT_FREETYPE_H
 #include <freetype/ftoutln.h>
 
 #include <libgnomeprint/gnome-print-private.h>
@@ -1528,6 +1526,7 @@ gf_pso_sprintf (GnomeFontPsObject *pso, const gchar * format, ...)
 	
 	oldlocale = g_strdup (setlocale (LC_NUMERIC, NULL));
 	setlocale (LC_NUMERIC, "C");
+#warning: We should not be using setlocale
 		
 	va_start (arguments, format);
 	text = g_strdup_vprintf (format, arguments);
