@@ -40,6 +40,7 @@
 #include <libgnomeprint/gnome-print-multipage.h>
 #include <libgnomeprint/gnome-print-job-private.h>
 #include <libgnomeprint/gnome-print-config-private.h>
+#include <libgnomeprint/gnome-print-i18n.h>
 
 typedef struct _JOBPrivate JOBPrivate;
 
@@ -65,6 +66,13 @@ struct _JOBPrivate {
 	GList *LY_LIST;
 };
 
+enum
+{
+  PROP_0,
+  PROP_CONFIG,
+};
+ 
+
 #define GNOME_PRINT_JOB_CLOSED(m) (((JOBPrivate *) (m)->priv)->closed)
 
 static void gnome_print_job_class_init (GnomePrintJobClass *klass);
@@ -74,10 +82,19 @@ static void gnome_print_job_finalize (GObject *object);
 static void job_update_layout_data (GnomePrintJob *job);
 static void job_parse_config_data (GnomePrintJob *job);
 static void job_clear_config_data (GnomePrintJob *job);
-static gboolean job_parse_transform (guchar *str, gdouble *transform);
+
+static void gnome_print_job_get_property (GObject      *object,
+                                          guint         prop_id,
+                                          GValue       *value,
+                                          GParamSpec   *pspec);
+static void gnome_print_job_set_property (GObject      *object,
+                                          guint         prop_id,
+                                          const GValue *value,
+                                          GParamSpec   *pspec);
 
 static GObjectClass *parent_class;
 
+
 GType
 gnome_print_job_get_type (void)
 {
@@ -103,6 +120,18 @@ gnome_print_job_class_init (GnomePrintJobClass *klass)
 	GObjectClass *object_class;
 	
 	object_class = (GObjectClass *) klass;
+
+	object_class->set_property = gnome_print_job_set_property;
+	object_class->get_property = gnome_print_job_get_property;
+
+	g_object_class_install_property (object_class,
+                                   PROP_CONFIG,
+                                   g_param_spec_object ("config",
+                                                        _("Job Configuration"),
+                                                        _("The configuration for the print job"),
+                                                        GNOME_TYPE_PRINT_CONFIG,
+                                                        G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+  
 	parent_class = g_type_class_peek_parent (klass);
 
 	object_class->finalize = gnome_print_job_finalize;
@@ -150,6 +179,55 @@ gnome_print_job_finalize (GObject *object)
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
+static void
+gnome_print_job_get_property (GObject *object, guint prop_id, GValue *value,
+	                     GParamSpec *pspec)
+{
+  GnomePrintJob *job = GNOME_PRINT_JOB (object);
+
+  switch (prop_id)
+    {
+    case PROP_CONFIG:
+      g_value_set_object (value, gnome_print_job_get_config (job));
+      break;
+  
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
+    }
+}
+
+static void
+gnome_print_job_set_property (GObject * object, guint prop_id,
+	                     const GValue * value, GParamSpec * pspec)
+{
+  GnomePrintJob *job = GNOME_PRINT_JOB (object);
+  GnomePrintConfig *config = NULL;
+
+  switch (prop_id)
+    {
+    case PROP_CONFIG:
+	    config = GNOME_PRINT_CONFIG (g_value_get_object (value));
+	    if (job->config) {
+		    gnome_print_config_unref (job->config);
+		    job->config = NULL;
+	    }
+	    
+	    if(config) {
+		    gnome_print_config_ref (config);
+		    job->config = config;
+	    } else {
+		    /* Use the default: */
+		    job->config = gnome_print_config_default ();
+	    }
+	    break;
+    default:
+	    G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+	    break;
+    }
+}
+
+
 /**
  * gnome_print_job_new:
  * @config: The job options, can be NULL in which case a
@@ -164,12 +242,7 @@ gnome_print_job_new (GnomePrintConfig *config)
 {
 	GnomePrintJob *job;
 	
-	job = g_object_new (GNOME_TYPE_PRINT_JOB, NULL);
-
-	if (config)
-		job->config = gnome_print_config_ref (config);
-	else
-		job->config = gnome_print_config_default ();
+	job = g_object_new (GNOME_TYPE_PRINT_JOB, "config", config, NULL);
 
 	return job;
 }
@@ -200,6 +273,8 @@ gnome_print_job_get_config (GnomePrintJob *job)
  * 
  * Retrieve the GnomePrintContext which applications
  * print to.
+ *
+ * The caller is responsible to unref the context when s/he is done with it.
  * 
  * Return value: The printing context, NULL on error
  **/
@@ -210,11 +285,11 @@ gnome_print_job_get_context (GnomePrintJob *job)
 	g_return_val_if_fail (GNOME_IS_PRINT_JOB (job), NULL);
 	g_return_val_if_fail (job->input_file == NULL, NULL);
 
-	if (job->meta)
-		g_object_ref (G_OBJECT (job->meta));
-	else
+	if (job->meta == NULL)
 		job->meta = gnome_print_meta_new ();
 
+	g_object_ref (G_OBJECT (job->meta));
+
 	return job->meta;
 }
 
@@ -792,7 +867,6 @@ job_parse_config_data (GnomePrintJob *job)
 	JOBPrivate *pp;
 	const GnomePrintUnit *unit;
 	GPANode *layout;
-	gchar *loc;
 
 	g_return_if_fail (job->priv);
 
@@ -804,8 +878,6 @@ job_parse_config_data (GnomePrintJob *job)
 
 	/* Now the fun part */
 
-	loc = g_strdup (setlocale (LC_NUMERIC, NULL));
-	setlocale (LC_NUMERIC, "C");
 
 	/* Physical size */
 	if (gnome_print_config_get_length (job->config, GNOME_PRINT_KEY_PAPER_WIDTH, &pp->pw, &unit)) {
@@ -848,7 +920,7 @@ job_parse_config_data (GnomePrintJob *job)
 						g_warning ("Could not fetch transfrom from %s\n", child_id);
 						break;
 					}
-					job_parse_transform (transform, affines + 6 * pagenum);
+					gnome_print_parse_transform (transform, affines + 6 * pagenum);
 					g_free (transform);
 					g_free (child_id);
 					pagenum += 1;
@@ -868,9 +940,6 @@ job_parse_config_data (GnomePrintJob *job)
 		art_affine_identity (pp->affines);
 		pp->num_affines = 1;
 	}
-
-	setlocale (LC_NUMERIC, loc);
-	g_free (loc);
 }
 
 #define A4_WIDTH (210 * 72 / 25.4)
@@ -909,139 +978,6 @@ job_clear_config_data (GnomePrintJob *job)
 
 /* FIXME: This function should not be here, we should not be including gpa-private.h too */
 
-/* All keys can be NULL */
-GnomePrintLayoutData *
-gnome_print_config_get_layout_data (GnomePrintConfig *config,
-				    const guchar *pagekey,
-				    const guchar *porientkey,
-				    const guchar *lorientkey,
-				    const guchar *layoutkey)
-{
-	GnomePrintLayoutData *lyd;
-	guchar key[1024];
-	const GnomePrintUnit *unit;
-	GPANode *layout;
-	gchar *loc;
-	/* Local data */
-	GnomePrintLayoutPageData *pages;
-	gdouble porient[6], lorient[6];
-	gdouble pw, ph, lyw, lyh;
-	gint num_pages;
-	gint numlp;
-	
-	g_return_val_if_fail (config != NULL, NULL);
-
-	if (!pagekey)
-		pagekey = GNOME_PRINT_KEY_PAPER_SIZE;
-	if (!porientkey)
-		porientkey = GNOME_PRINT_KEY_PAPER_ORIENTATION;
-	if (!lorientkey)
-		lorientkey = GNOME_PRINT_KEY_PAGE_ORIENTATION;
-	if (!layoutkey)
-		layoutkey = GNOME_PRINT_KEY_LAYOUT;
-
-
-	/* Initialize */
-	pw = 210 * 72.0 / 25.4;
-	ph = 297 * 72.0 / 25.4;
-	art_affine_identity (porient);
-	art_affine_identity (lorient);
-	lyw = 1.0;
-	lyh = 1.0;
-	num_pages = 0;
-	pages = NULL;
-
-	/* Now the fun part */
-
-	loc = g_strdup (setlocale (LC_NUMERIC, NULL));
-	setlocale (LC_NUMERIC, "C");
-
-	/* Physical size */
-	g_snprintf (key, 1024, "%s.Width", pagekey);
-	if (gnome_print_config_get_length (config, key, &pw, &unit)) {
-		gnome_print_convert_distance (&pw, unit, GNOME_PRINT_PS_UNIT);
-	}
-	g_snprintf (key, 1024, "%s.Height", pagekey);
-	if (gnome_print_config_get_length (config, key, &ph, &unit)) {
-		gnome_print_convert_distance (&ph, unit, GNOME_PRINT_PS_UNIT);
-	}
-	/* Physical orientation */
-	g_snprintf (key, 1024, "%s.Paper2PrinterTransform", porientkey);
-	gnome_print_config_get_transform (config, key, porient);
-	/* Logical orientation */
-	g_snprintf (key, 1024, "%s.Page2LayoutTransform", lorientkey);
-	gnome_print_config_get_transform (config, key, lorient);
-	/* Layout size */
-	g_snprintf (key, 1024, "%s.Width", layoutkey);
-	gnome_print_config_get_double (config, key, &lyw);
-	g_snprintf (key, 1024, "%s.Height", layoutkey);
-	gnome_print_config_get_double (config, key, &lyh);
-
-	/* Get the layout tree from the config database */
-	layout = gpa_node_get_child_from_path (GNOME_PRINT_CONFIG_NODE (config), layoutkey);
-	if (!layout) {
-		layout = gpa_node_get_child_from_path (NULL, "Globals.Document.Page.Layout.Plain");
-		if (!layout) {
-			g_warning ("Could not get Globals.Document.Page.Layout.Plain");
-			return NULL;
-		}
-	}
-
-	/* Now come the affines */
-	numlp = 0;
-	if (gpa_node_get_int_path_value (layout, "LogicalPages", &numlp) && (numlp > 0)) {
-		GPANode *pnodes;
-		pnodes = gpa_node_get_child_from_path (layout, "Pages");
-		if (pnodes) {
-			GPANode *page;
-			gint pagenum;
-			pages = g_new (GnomePrintLayoutPageData, numlp);
-			pagenum = 0;
-			for (page = gpa_node_get_child (pnodes, NULL); page != NULL; page = gpa_node_get_child (pnodes, page)) {
-				guchar *transform;
-				transform = gpa_node_get_value (page);
-				gpa_node_unref (page);
-				if (!transform)
-					break;
-				job_parse_transform (transform, pages[pagenum].matrix);
-				g_free (transform);
-				pagenum += 1;
-				if (pagenum >= numlp)
-					break;
-			}
-			gpa_node_unref (pnodes);
-			if (pagenum == numlp) {
-				num_pages = numlp;
-			} else {
-				g_free (pages);
-			}
-		}
-	}
-	gpa_node_unref (layout);
-
-	setlocale (LC_NUMERIC, loc);
-	g_free (loc);
-	
-	if (num_pages == 0) {
-		g_warning ("Could not get_layout_data\n");
-		return NULL;
-	}
-
-	/* Success */
-
-	lyd = g_new (GnomePrintLayoutData, 1);
-	lyd->pw = pw;
-	lyd->ph = ph;
-	memcpy (lyd->porient, porient, 6 * sizeof (gdouble));
-	memcpy (lyd->lorient, lorient, 6 * sizeof (gdouble));
-	lyd->lyw = lyw;
-	lyd->lyh = lyh;
-	lyd->num_pages = num_pages;
-	lyd->pages = pages;
-
-	return lyd;
-}
-
 void
 gnome_print_layout_data_free (GnomePrintLayoutData *lyd)
 {
@@ -1172,8 +1108,8 @@ gnome_print_layout_free (GnomePrintLayout *layout)
 	g_free (layout);
 }
 
-static gboolean
-job_parse_transform (guchar *str, gdouble *transform)
+gboolean
+gnome_print_parse_transform (guchar *str, gdouble *transform)
 {
 	gdouble t[6];
 	guchar *p;
@@ -1204,7 +1140,7 @@ job_parse_transform (guchar *str, gdouble *transform)
 			t[i] = -M_SQRT1_2;
 			e = p + 8;
 		} else {
-			t[i] = strtod (p, &e);
+			t[i] = g_ascii_strtod (p, &e);
 		}
 		if (e == (gchar *) p)
 			return FALSE;
@@ -1242,38 +1178,3 @@ gnome_print_job_print_to_file (GnomePrintJob *job, gchar *output)
 	return GNOME_PRINT_OK;
 }
 
-/* fixme: Move this (and previous) to gnome-print-config.c (Lauris) */
-
-gboolean
-gnome_print_config_get_transform (GnomePrintConfig *config, const guchar *key, gdouble *transform)
-{
-	guchar *v;
-	gdouble t[6];
-	gboolean ret;
-	gchar *loc;
-
-	g_return_val_if_fail (config != NULL, FALSE);
-	g_return_val_if_fail (key != NULL, FALSE);
-	g_return_val_if_fail (*key != '\0', FALSE);
-	g_return_val_if_fail (config != NULL, FALSE);
-
-	v = gnome_print_config_get (config, key);
-	if (!v) {
-		return FALSE;
-	}
-
-	loc = g_strdup (setlocale (LC_NUMERIC, NULL));
-	setlocale (LC_NUMERIC, "C");
-
-	ret = job_parse_transform (v, t);
-	g_free (v);
-	if (ret) {
-		memcpy (transform, t, 6 * sizeof (gdouble));
-	}
-
-	setlocale (LC_NUMERIC, loc);
-	g_free (loc);
-
-	return ret;
-}
-
