@@ -133,9 +133,6 @@ void gnome_print_layout_data_free (GnomePrintLayoutData *layoutdata);
 GnomePrintLayout *gnome_print_layout_new_from_data (const GnomePrintLayoutData *layoutdata);
 void gnome_print_layout_free (GnomePrintLayout *layout);
 
-/* This method first appeared in 1.109 CVS */
-gboolean gnome_print_config_get_transform (GnomePrintConfig *config, const guchar *key, gdouble *transform);
-
 
 
 /* I'm putting this util funcs here for the mean time, find
@@ -147,6 +144,7 @@ struct _GnomePrintBuffer {
 };
 void gnome_print_buffer_munmap (GnomePrintBuffer *b);
 gint gnome_print_buffer_mmap   (GnomePrintBuffer *b, const guchar *file_name);
+gboolean gnome_print_parse_transform (guchar *str, gdouble *transform);
 /* End */
 
 G_END_DECLS
